
package imageConvolution;

import imageConvolution.logic.Convolution;
import imageConvolution.ui.ImagesFilter;
import imageConvolution.ui.MainWindow;
import imageConvolution.ui.functions.CopyEdgeFunction;
import imageConvolution.ui.functions.NormalizeFunction;
import imageConvolution.ui.functions.OpenImageFunction;
import imageConvolution.ui.functions.ProcessFunction;
import imageConvolution.ui.functions.ResetSourceFunction;
import imageConvolution.ui.functions.SaveImageFunction;
import imageConvolution.ui.functions.UseAsSourceFunction;
import imageConvolution.ui.utils.SwingWorker;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * The program flow component that controls the main flow of the application
 * such as start (shows GUI), open file, save file and exit, etc.
 * 
 * @author TineL
 * @version 1.0, 5.1.2007
 * @since 1.0
 */
public class ProgramFlow {

  /** The singleton instance. */
  private static ProgramFlow instance=null;

  /**
   * @return the singleton instance of this class (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  public static ProgramFlow getInstance() {
    if (ProgramFlow.instance==null) ProgramFlow.instance=new ProgramFlow();
    return ProgramFlow.instance;
  }

  /** The main window. */
  private MainWindow mainWindow;

  /** The logic. */
  private BufferedImage openedImage;
  private ImagesFilter imageFileFilter;
  private File lastSelectedOpenFile;
  private File lastSelectedSaveFile;

  /**
   * Starts the application. Shows the GUI.
   * 
   * @since 1.0
   */
  public void start() {
    System.out.println("ProgramFlow.start: Starting application...");

    this.mainWindow=new MainWindow();

    SaveImageFunction.getInstance().setEnabled(false);
    ProcessFunction.getInstance().setEnabled(false);
    UseAsSourceFunction.getInstance().setEnabled(false);
    ResetSourceFunction.getInstance().setEnabled(false);

    this.imageFileFilter=new ImagesFilter("Image Files");
    getImageFileFilter().addExtensions(ImageIO.getReaderFormatNames());
    getMainWindow().getStatusBar().setProcessMessage("Ready");

    // openImage(new File("fruit.jpg"));

    System.out.println("ProgramFlow.start: Application started.");
  }

  /**
   * Shows the open file dialog. If the file is selected, the file opening
   * process begins.
   * 
   * @since 1.0
   */
  public void showOpenFileDialog() {
    JFileChooser fileChooser=new JFileChooser();
    fileChooser.addChoosableFileFilter(getImageFileFilter());
    fileChooser.setCurrentDirectory(getLastSelectedOpenFile());

    int returnValue=fileChooser.showOpenDialog(getMainWindow());

    if (returnValue==JFileChooser.APPROVE_OPTION) {
      setLastSelectedOpenFile(fileChooser.getSelectedFile());
      openImage(fileChooser.getSelectedFile());
    }
  }

  /**
   * Shows the open file dialog. If the file is selected, the file opening
   * process begins.
   * 
   * @since 1.0
   */
  public void showSaveFileDialog() {
    JFileChooser fileChooser=new JFileChooser();
    fileChooser.addChoosableFileFilter(getImageFileFilter());
    fileChooser.setCurrentDirectory(getLastSelectedSaveFile()==null
      ?getLastSelectedOpenFile():getLastSelectedSaveFile());

    int returnValue=fileChooser.showSaveDialog(getMainWindow());

    if (returnValue==JFileChooser.APPROVE_OPTION) {
      setLastSelectedSaveFile(fileChooser.getSelectedFile());
      saveImage(getMainWindow().getChangedPane().getImage(), fileChooser
        .getSelectedFile());
    }
  }

  private void openImage(final File image) {
    getMainWindow().setWaitCursor(true);
    OpenImageFunction.getInstance().setEnabled(false);
    ProcessFunction.getInstance().setEnabled(false);
    ResetSourceFunction.getInstance().setEnabled(false);
    UseAsSourceFunction.getInstance().setEnabled(false);

    // Thread
    final SwingWorker worker=new SwingWorker() {
      public Object construct() {
        try {
          setOpenedImage(ImageIO.read(image));

        } catch (IOException e) {
          e.printStackTrace();
          JOptionPane.showMessageDialog(getMainWindow(),
            "A file read error has occured.", "Read Error",
            JOptionPane.ERROR_MESSAGE);

        } catch (IllegalArgumentException e) {
          e.printStackTrace();
        }
        return null;
      }

      public void finished() {
        OpenImageFunction.getInstance().setEnabled(true);
        ProcessFunction.getInstance().setEnabled(true);
        ResetSourceFunction.getInstance().setEnabled(true);
        UseAsSourceFunction.getInstance().setEnabled(true);
        getMainWindow().getStatusBar().setProcessMessage("Ready");
        getMainWindow().setWaitCursor(false);

        if (getOpenedImage()==null) {
          JOptionPane.showMessageDialog(getMainWindow(), "The file '"
            +image.getAbsolutePath()+"' is not supported image format.",
            "Not Supported", JOptionPane.ERROR_MESSAGE);
          showOpenFileDialog();

        } else {
          getMainWindow().getOriginalPane().setImage(getOpenedImage());
          getMainWindow().getStatusBar().setFileDescription(
            image.getAbsolutePath());
        }
      }
    };

    // Start thread
    worker.start();
    getMainWindow().getStatusBar().setProcessMessage("Opening image...");
  }

  private void saveImage(final BufferedImage image, final File file) {
    final String extension=ImagesFilter.getFileExtension(file);
    if (extension==null) {
      JOptionPane.showMessageDialog(getMainWindow(),
        "Please specify an image format with the correct file extension.",
        "Not Specified", JOptionPane.ERROR_MESSAGE);
      showSaveFileDialog();
      return;
    }

    List writers=Arrays.asList(ImageIO.getWriterFormatNames());
    if (!writers.contains(extension)) {
      JOptionPane.showMessageDialog(getMainWindow(), "The image format '"
        +extension+"' is not supported.", "Not Supported",
        JOptionPane.ERROR_MESSAGE);
      showSaveFileDialog();
      return;
    }

    if (file.exists()) {
      int res=JOptionPane.showConfirmDialog(getMainWindow(),
        "The image already exists.\nWould you like to overwrite it?",
        "Image Exists", JOptionPane.YES_NO_CANCEL_OPTION);

      switch (res) {
      case JOptionPane.NO_OPTION:
        showSaveFileDialog();
      case JOptionPane.CANCEL_OPTION:
        return;
      }
    }

    getMainWindow().setWaitCursor(true);
    SaveImageFunction.getInstance().setEnabled(false);

    // Thread
    final SwingWorker worker=new SwingWorker() {
      public Object construct() {
        try {
          ImageIO.write(image, extension, file);

        } catch (IOException e) {
          e.printStackTrace();
          JOptionPane.showMessageDialog(getMainWindow(),
            "A file write error has occured.", "Write Error",
            JOptionPane.ERROR_MESSAGE);

        } catch (IllegalArgumentException e) {
          e.printStackTrace();
        }
        return null;
      }

      public void finished() {
        SaveImageFunction.getInstance().setEnabled(true);
        getMainWindow().getStatusBar().setProcessMessage("Ready");
        getMainWindow().setWaitCursor(false);
      }
    };

    // Start thread
    worker.start();
    getMainWindow().getStatusBar().setProcessMessage("Saving image...");
  }

  public void processImage() {
    getMainWindow().setWaitCursor(true);
    OpenImageFunction.getInstance().setEnabled(false);
    SaveImageFunction.getInstance().setEnabled(false);
    ProcessFunction.getInstance().setEnabled(false);
    ResetSourceFunction.getInstance().setEnabled(false);
    UseAsSourceFunction.getInstance().setEnabled(false);

    // Thread
    final SwingWorker worker=new SwingWorker() {
      public Object construct() {
        BufferedImage img=Convolution.convolve(getMainWindow()
          .getOriginalPane().getImage(), getMainWindow().getOperationPane()
          .getMask(), NormalizeFunction.getInstance().isOn(), CopyEdgeFunction
          .getInstance().isOn());
        return img;
      }

      public void finished() {
        BufferedImage img=(BufferedImage)get();
        getMainWindow().getChangedPane().setImage(img);

        OpenImageFunction.getInstance().setEnabled(true);
        SaveImageFunction.getInstance().setEnabled(img!=null);
        ProcessFunction.getInstance().setEnabled(true);
        ResetSourceFunction.getInstance().setEnabled(true);
        UseAsSourceFunction.getInstance().setEnabled(img!=null);
        getMainWindow().getStatusBar().setProcessMessage("Ready");
        getMainWindow().setWaitCursor(false);
      }
    };

    // Start thread
    worker.start();
    getMainWindow().getStatusBar().setProcessMessage("Processing image...");
  }

  public void useProcessedImageAsSource() {
    BufferedImage bi=getMainWindow().getChangedPane().getImage();
    getMainWindow().getOriginalPane().setImage(bi);
    getMainWindow().getChangedPane().setImage(null);
    SaveImageFunction.getInstance().setEnabled(false);
    UseAsSourceFunction.getInstance().setEnabled(false);
  }

  public void resetSourceImage() {
    getMainWindow().getOriginalPane().setImage(getOpenedImage());
  }

  /**
   * Exits the application in a normal way.
   * 
   * @since 1.0
   */
  public void exit() {
    System.out.println("ProgramFlow.exit: Application exit.");
    System.exit(0);
  }

  /**
   * @return the main window.
   * @since 1.0
   */
  public MainWindow getMainWindow() {
    return this.mainWindow;
  }

  /**
   * @return the lastSelectedFile.
   * @since 1.0
   */
  private File getLastSelectedOpenFile() {
    return this.lastSelectedOpenFile;
  }

  /**
   * @param lastSelectedFile the lastSelectedFile.
   * @since 1.0
   */
  private void setLastSelectedOpenFile(File lastSelectedFile) {
    this.lastSelectedOpenFile=lastSelectedFile;
  }

  /**
   * @return the filter.
   * @since 1.0
   */
  private ImagesFilter getImageFileFilter() {
    return this.imageFileFilter;
  }

  /**
   * @return the lastSelectedSaveFile.
   * @since 1.0
   */
  private File getLastSelectedSaveFile() {
    return this.lastSelectedSaveFile;
  }

  /**
   * @param lastSelectedSaveFile the lastSelectedSaveFile.
   * @since 1.0
   */
  private void setLastSelectedSaveFile(File lastSelectedSaveFile) {
    this.lastSelectedSaveFile=lastSelectedSaveFile;
  }

  /**
   * @return the openedImage.
   * @since 1.0
   */
  private BufferedImage getOpenedImage() {
    return this.openedImage;
  }

  /**
   * @param openedImage the openedImage.
   * @since 1.0
   */
  private void setOpenedImage(BufferedImage openedImage) {
    this.openedImage=openedImage;
  }
}
