/*
 * Copyright 2004-2006 Autronic d.o.o. Slovenia
 * 
 * You may not use this file except with explicit written permission
 * of Autronic d.o.o. Slovenia.
 */

package imageConvolution.ui.functions;

import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Iterator;

import javax.swing.AbstractButton;

/**
 * The abstract class that represents the switch function (the function that has
 * two states: on and off) associated to an any type of the Swing
 * {@link javax.swing.AbstractButton}. This class collects all buttons that
 * have the same functionality. The function constist of the action, performed
 * when the button's event occures, two function states on and off that
 * reflects the buttons' "on" state (selected, checked or toggled and enabled)
 * and the function's enable state that reflects to the enable state of the
 * buttons.<br>
 * <br>
 * The abstract method <i>run</i> will be automatically called, when an event
 * occures.<br>
 * To programmatically perform a button's event, call <i>switchOn</i> method.
 *
 * @author TineL
 * @version 1.4, 16.3.2006
 * @since 1.4
 */
public abstract class ButtonSwitchFunction extends ButtonFunction implements
  ItemListener {

  /*
   * TODO: If explicitly set to on=false, it overrides the init() function in
   * the Function abstract class constructor, because this parameter is set
   * after the constructor is being called and already exists before the
   * call to constructor! 
   * So, if you call setOn(true) in the init() method of your function (which
   * set this parameter to true), it is overriden after the init() method.
   * If this parameter is set to nothing private boolean on; than it's value
   * is not overriden. The private boolean on=isOn(); also works :).
   * The default value (if not set by setOn()) is false.
   * 
   * Explore this weird behaviour (Java bug?) - see OverrideState project!
   */
  /** The function switch state. */
  private boolean on;

  /**
   * Associate the button to this function.
   * 
   * @param button the button (<code>null</code> not permitted).
   * @since 1.4
   */
  public void addButton(AbstractButton button) {
    super.addButton(button);
    button.setSelected(isOn());
    button.addItemListener(this);
  }

  /**
   * Remove the button from this function. 
   * 
   * @param button the button (<code>null</code> not permitted).
   * @since 1.4
   */
  public void removeButton(AbstractButton button) {
    button.removeItemListener(this);
    super.removeButton(button);
  }

  /**
   * Runs this function if <i>on</i> argument differs from the function's
   * current switch state and if this function is enabled. If it differs, it
   * sets this function to the defined switch state. If not so, nothing is
   * done.<br>
   * Useful to programmatically perform switch.
   * 
   * @param on <code>true</code> if switch to on, <code>false</code> if switch
   * to off.
   * @since 1.4
   */
  public void switchOn(boolean on) {
    if (!isEnabled()) return;
    if (isOn()==on) return;
    setOn(on);
    run();
    save();
  }

  /**
   * Sets this function to the defined switch state. At the same time it sets
   * selected, checked or toggled state to all associated buttons.<br>
   * Useful to initialize function's switch state.<br>
   * <br>
   * Note: Function is not run. To run the function, call <i>run</i> method. 
   * 
   * @param on <code>true</code> if set to on, <code>false</code> if set to off.
   * @since 1.4
   */
  public void setOn(boolean on) {
    this.on=on;

    Iterator buttonsI=getComponents().iterator();
    while (buttonsI.hasNext()) {
      AbstractButton button=(AbstractButton)buttonsI.next();
      button.removeItemListener(this);
      button.setSelected(isOn());
      button.addItemListener(this);
    }
  }

  /**
   * Returns the function's switch state.
   * 
   * @return <code>true</code> if function is on, <code>false</code> if off.
   * @since 1.4
   */
  public boolean isOn() {
    return this.on;
  }

  /**
   * When the function's state is being changed, this method is called to save
   * the new state to the settings.
   * 
   * @since 1.4
   */
  protected abstract void save();

  /**
   * @since 1.4
   */
  public void actionPerformed(ActionEvent e) {
    super.actionPerformed(e);
    save();
  }

  /**
   * @since 1.4
   */
  public void itemStateChanged(ItemEvent e) {
    int state=e.getStateChange();
    if (state==ItemEvent.SELECTED||state==ItemEvent.DESELECTED)
      setOn(state==ItemEvent.SELECTED);
  }
}
