/*
 * Copyright 2004-2006 Autronic d.o.o. Slovenia
 * 
 * You may not use this file except with explicit written permission
 * of Autronic d.o.o. Slovenia.
 */

package imageConvolution.ui.functions;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;

/**
 * The abstract class that represents the function associated to an any type of
 * the Swing {@link javax.swing.JComponent}. This class collects all components
 * that have the same functionality. The function constist function's enable
 * state that reflects to the enable state of the component.<br>
 * <br>
 * The abstract method <i>run</i> will be automatically called, when an event
 * occures.
 *
 * @author TineL
 * @version 1.4, 16.3.2006
 * @since 1.4
 */
public abstract class Function implements PropertyChangeListener {

  /** The list of associated components. */
  private List components=new ArrayList();

  /** The function enable state. */
  private boolean enabled=true;

  /**
   * Creates <code>Function</code> and initializes it.
   *
   * @since 1.4
   */
  public Function() {
    init();
  }

  /**
   * Associate the components to this function.
   * 
   * @param component the component (<code>null</code> not permitted).
   * @since 1.4
   */
  protected void addComponent(JComponent component) {
    getComponents().add(component);
    component.setEnabled(isEnabled());
    component.addPropertyChangeListener(this);
  }

  /**
   * Remove the component from this function. 
   * 
   * @param component the component (<code>null</code> not permitted).
   * @since 1.4
   */
  protected void removeComponent(JComponent component) {
    component.removePropertyChangeListener(this);
    getComponents().remove(component);
  }

  /**
   * When this function is being constructed, this method is called to set
   * function's default state.
   * 
   * @since 1.4
   */
  protected abstract void init();

  /**
   * A component's event will call this method to perform this function's
   * action.
   * 
   * @since 1.4
   */
  protected abstract void run();

  /**
   * Sets this function to the defined enable state. At the same time it sets
   * enable state to all associated components. 
   * 
   * @param enabled <code>true</code> if set to enabled, <code>false</code> if
   * set to disabled.
   * @since 1.4
   */
  public void setEnabled(boolean enabled) {
    this.enabled=enabled;

    Iterator componentsI=getComponents().iterator();
    while (componentsI.hasNext()) {
      JComponent component=(JComponent)componentsI.next();
      component.removePropertyChangeListener(this);
      component.setEnabled(enabled);
      component.addPropertyChangeListener(this);
    }
  }

  /**
   * @return <code>true</code> if function is enabled, <code>false</code> if
   * disabled.
   * @since 1.4
   */
  public boolean isEnabled() {
    return this.enabled;
  }

  /**
   * @return the components list (<code>null</code> not possible).
   * @since 1.4
   */
  protected List getComponents() {
    return this.components;
  }

  /**
   * @since 1.4
   */
  public void propertyChange(PropertyChangeEvent e) {
    if (e.getPropertyName().equals("enabled"))
      setEnabled(((Boolean)e.getNewValue()).booleanValue());
  }
}
